@@ -21,6 +21,8 @@
 #include <signal.h>
 #include <sys/wait.h>
 #include <stdio.h>
+#include <ctype.h>
+#include <string.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -28,16 +30,20 @@
 #include <gpe/init.h>
 #include <gpe/pixmaps.h>
 #include <gpe/errorbox.h>
-#include <gpe/tray.h>
+//#include <gpe/tray.h>
+#include <libmb/mb.h>
 #include <gpe/popup.h>
 #include <gpe/spacing.h>
 
 #include "main.h"
+#include "netlink.h"
 
 #define _(x) gettext(x)
 
 #define BIN_INFO PREFIX "/bin/gpe-info"
 #define BIN_CONFIG PREFIX "/bin/gpe-conf"
+#define BIN_PING "/bin/ping"
+
 #define PARAM_INFO "network"
 #define PARAM_CONFIG "network"
 
@@ -49,12 +55,30 @@ struct gpe_icon my_icons[] = {
 };
 
 
-static GtkWidget *icon;
+MBPixbuf      *Pixbuf;
+MBPixbufImage *AppImage;
+MBTrayApp *app = NULL;
+
+typedef enum
+{
+	ICON_NET_OFF,
+	ICON_NET_ON,
+	NUM_ICONS
+}
+n_images;
+
+MBPixbufImage *net_status_icons[NUM_ICONS];
 
 static GtkWidget *menu;
 
 gboolean net_is_on = FALSE;
-GtkWidget *dock_window;
+
+static int netlink_fd;
+
+/* Iface   Destination     Gateway         Flags   RefCnt  Use     Metric  Mask   MTU      Window  IRTT
+   eth0    0105A8C0        0102A8C0        0006    0       0       0       FFFFFFF0                     */
+
+#define RTF_UP  0x1
 
 static gboolean
 net_get_status()
@@ -62,19 +86,76 @@ net_get_status()
 	FILE *pipe;
 	char buffer[256];
 	int result = FALSE;
+	char gw[9];
+	char *ip = NULL;
+	int ipdigit[4];
 	
-	pipe = popen ("/bin/netstat -rn", "r");
+	pipe = fopen ("/proc/net/route", "r");
 
-	if (pipe > 0)
-    {
+	if (pipe)
+	{
   		while ((feof(pipe) == 0))
-    	{
-      		fgets (buffer, 255, pipe);
-			if (g_str_has_prefix(buffer,"0.0.0.0") || g_str_has_prefix(buffer,"default"))
-			  result = TRUE;
+		{
+			if (fgets (buffer, 255, pipe))
+			{
+				char *p, *q;
+				int flags;
+
+				p = strchr (buffer, '\t');
+				if (!p)
+					continue;
+				while (isspace (*p))
+					p++;
+				q = strchr (p, '\t');
+				if (!q)
+					continue;
+				*q++ = 0;
+				if (strcmp (p, "00000000"))
+					continue;	// Not default route
+				while (isspace (*q))
+					q++;
+				strncpy(gw,q,8); // copy ip of gw
+				gw[8] = 0;
+				p = strchr (q, '\t');	// Skip over gateway
+				if (!p)
+					continue;
+				while (isspace (*p))
+					p++;
+				q = strchr (p, '\t');
+				if (!q)
+					continue;
+				*q = 0;
+				if (!sscanf (p, "%x", &flags))
+					continue;
+				if (flags & RTF_UP)
+				{
+					result = TRUE;
+					break;
+				}
+			}
 		}
 		pclose(pipe);		
 	}
+	/* check if we can ping our default gw */
+	if (result)
+	{
+		sscanf(gw,"%02x%02x%02x%02x",
+			&ipdigit[0],&ipdigit[1],&ipdigit[2],&ipdigit[3]);
+		ip = g_strdup_printf("%s -c 1 -q %i.%i.%i.%i",
+			BIN_PING,ipdigit[3],ipdigit[2],ipdigit[1],ipdigit[0]);
+
+		pipe = popen (ip, "r");
+		if (pipe > 0)
+		{
+			while ((feof(pipe) == 0))
+			{
+				fgets (buffer, 255, pipe);
+				if (strstr(buffer,"100%"))
+				  result = FALSE;
+			}
+			pclose(pipe);		
+		}		
+	}
 	return result;
 }
 
@@ -82,12 +163,12 @@ net_get_status()
 static gboolean
 remove_dock_message (guint id)
 {
-	gpe_system_tray_cancel_message (dock_window->window, id);
+//	gpe_system_tray_cancel_message (GDK_WINDOW(), id);
 	return FALSE;
 }
 
 
-static gboolean
+gboolean
 update_netstatus (void)
 {
 	GdkBitmap *bitmap;
@@ -98,24 +179,15 @@ update_netstatus (void)
 	
 	if (net_is_on != oldstatus)
 	{
-		if (net_is_on)
-			gtk_image_set_from_pixbuf (GTK_IMAGE (icon),
-				   gpe_find_icon ("net-on"));
-		else
-			gtk_image_set_from_pixbuf (GTK_IMAGE (icon),
-				   gpe_find_icon ("net-off"));
-		gdk_pixbuf_render_pixmap_and_mask (
-			gpe_find_icon (net_is_on ? "net-on" : "net-off"), NULL,
-					   &bitmap, 128);
-		gtk_widget_shape_combine_mask (dock_window, bitmap, 0, 0);
-		gdk_bitmap_unref (bitmap);
-		msg_id = gpe_system_tray_send_message (dock_window->window, 
+		mb_tray_app_repaint (app);
+/*		msg_id = gpe_system_tray_send_message (GDK_WINDOW(), 
 			net_is_on ? _("Network connection established.") 
 				: _("Network connection lost.")
 			, 0);
+		
 		g_timeout_add (10000, (GSourceFunc) remove_dock_message,
 		       (gpointer) msg_id);
-	}
+*/	}
 	return TRUE;
 }
 
@@ -146,6 +218,7 @@ do_network_info (void)
 	} 
 }
 
+
 static void
 do_network_config (void)
 {
@@ -173,23 +246,74 @@ sigterm_handler (int sig)
 
 
 static void
-clicked (GtkWidget * w, GdkEventButton * ev)
+clicked (MBTrayApp *app, int x, int y, Bool is_released )
+//clicked (GtkWidget * w, GdkEventButton * ev)
 {
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, gpe_popup_menu_position,
-			w, ev->button, ev->time);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL/*gpe_popup_menu_position*/,
+		NULL,0, gtk_get_current_event_time());
+			//w, ev->button, ev->time);
+}
+
+
+static gboolean
+rtnl_callback (GIOChannel *source, GIOCondition cond, gpointer data)
+{
+	rtnl_process ((int)data);
+
+	return TRUE;
+}
+
+
+void
+rtnl_connect_glib (int fd)
+{
+	GIOChannel *chan;
+
+	chan = g_io_channel_unix_new (fd);
+
+	g_io_add_watch (chan, G_IO_IN, rtnl_callback, (gpointer)fd);
+}
+
+
+void 
+paint_callback ( MBTrayApp *app, Drawable drw ) 
+{ 
+	MBPixbufImage *img_scaled;
+	MBPixbufImage *img_backing = NULL;
+	int use_index = net_is_on ? ICON_NET_ON : ICON_NET_OFF;
+  
+	img_backing = mb_tray_app_get_background (app, Pixbuf);
+
+	mb_pixbuf_img_composite(Pixbuf, img_backing, 
+			  net_status_icons[use_index], 
+			  0, 0);
+	mb_pixbuf_img_render_to_drawable(Pixbuf, img_backing, drw, 0, 0);
+	mb_pixbuf_img_free( Pixbuf, img_backing );
+}
+
+
+GdkFilterReturn
+event_filter (GdkXEvent *xev, GdkEvent *gev, gpointer data)
+{
+	XEvent    *ev  = (XEvent *)xev;
+	MBTrayApp *app = (MBTrayApp*)data;
+	Display *dpy = ev->xany.display;
+	mb_tray_handle_xevent (app, ev); 
+	return GDK_FILTER_CONTINUE;
 }
 
 
 int
 main (int argc, char *argv[])
 {
-	Display *dpy;
-	GtkWidget *window;
-	GdkBitmap *bitmap;
+//	Display *dpy;
+//	GtkWidget *window;
+//	GdkBitmap *bitmap;
 	GtkWidget *menu_remove;
 	GtkWidget *menu_config;
 	GtkWidget *menu_info;
 	GtkTooltips *tooltips;
+	GdkPixbuf *pixmap;
 
 	if (gpe_application_init (&argc, &argv) == FALSE)
 		exit (1);
@@ -200,12 +324,23 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset (PACKAGE, "UTF-8");
 	textdomain (PACKAGE);
 
-	window = gtk_plug_new (0);
+	netlink_fd = rtnl_open ();
+	if (netlink_fd < 0)
+	  {
+	    gpe_error_box (_("Couldn't open netlink device"));
+	    exit (1);
+	  }
+
+	rtnl_connect_glib (netlink_fd);
+
+	app = mb_tray_app_new_with_display (_("Network Control"), NULL, paint_callback, &argc, &argv, GDK_DISPLAY() );
+	  
+/*	window = gtk_plug_new (0);
 	gtk_widget_set_usize (window, 16, 16);
 	gtk_widget_realize (window);
 
 	gtk_window_set_title (GTK_WINDOW (window), _("Network Control"));
-
+*/
 	signal (SIGTERM, sigterm_handler);
 
 	menu = gtk_menu_new ();
@@ -236,7 +371,22 @@ main (int argc, char *argv[])
 		exit (1);
 
 	net_is_on = net_get_status();
-	
+
+
+	Pixbuf = mb_pixbuf_new(mb_tray_app_xdisplay(app), mb_tray_app_xscreen(app));
+    pixmap = gpe_find_icon("net-on");	
+	net_status_icons[ICON_NET_ON] = mb_pixbuf_img_new_from_data (Pixbuf, gdk_pixbuf_get_pixels(pixmap), 
+		gdk_pixbuf_get_width(pixmap), gdk_pixbuf_get_height(pixmap), gdk_pixbuf_get_has_alpha(pixmap));
+    pixmap = gpe_find_icon("net-off");	
+	net_status_icons[ICON_NET_OFF] = mb_pixbuf_img_new_from_data (Pixbuf, gdk_pixbuf_get_pixels(pixmap), 
+		gdk_pixbuf_get_width(pixmap), gdk_pixbuf_get_height(pixmap), gdk_pixbuf_get_has_alpha(pixmap));
+
+	mb_tray_app_set_button_callback (app, clicked);
+	mb_tray_app_main_init(app);
+
+	gdk_window_add_filter (NULL, event_filter, (gpointer)app );
+
+/*	mb_tray_app_main(app);	
 	icon = gtk_image_new_from_pixbuf (
 			gpe_find_icon(net_is_on ? "net-on" :
 				"net-off")); 
@@ -267,9 +417,7 @@ main (int argc, char *argv[])
 
 	dock_window = window;
 	gpe_system_tray_dock (window->window);
-
-	gtk_timeout_add (2000, (GtkFunction) update_netstatus, NULL);
-
+*/
 	gtk_main ();
 	
 	exit (0);
